#include <linux/config.h>
#include <linux/module.h>
#include <linux/types.h>
#include <linux/kernel.h>
#include <asm/io.h>
#include <linux/mtd/mtd.h>
#include <linux/mtd/map.h>
#include <linux/mtd/cfi.h>
#include <linux/mtd/partitions.h>
#include <linux/mtd/concat.h>

#define WINDOW_ADDR_0   0xBFC00000
#define WINDOW_SIZE_0   0x00400000
#define WINDOW_ADDR_1   0xBF000000
#define WINDOW_SIZE_1   0x00400000

static struct map_info dummy_mips_map[2] = {
  {
    .name       = "DUMMY boot flash",
    .phys       = WINDOW_ADDR_0,
    .size       = WINDOW_SIZE_0,
  },
  {
    .name       = "Dummy non boot flash",
    .phys       = WINDOW_ADDR_1,
    .size       = WINDOW_SIZE_1,
  }
};

static struct mtd_partition boot_flash_partitions [] = {
  {
    .name    = "BOOT",
    .offset  = 0,
    .size    = 0x00010000,
  },
  {
    .name    = "JFFS2",
    .offset  = 0x00010000,
    .size    = 0x003f0000,
  },
};
 
/* 
 * Following structure holds the mtd_info pointers for the 
 * partitons we will be concatenating 
 */
static struct mtd_info *concat_partitions[2];

/* 
 * Following structure holds the mtd_info structure pointers for    
 * each of the flash devices 
 */
static struct mtd_info * mymtd[2], *concat_mtd;

//Listing 4-6 init_dummy_mips_mtd_bsp function
int __init init_dummy_mips_mtd_bsp (void)
{

  /* First probe for the boot flash */
  dummy_mips_map[0].virt = 
    (unsigned long)ioremap( 
                   dummy_mips_map[0].phys,dummy_mips_map[0].size);
  simple_map_init(&dummy_mips_map[0]);
  mymtd[0] = do_map_probe("cfi_probe", &dummy_mips_map[0]);
  if(mymtd[0])
    mymtd[0]->owner = THIS_MODULE;

    /* Repeat for the second flash */
  dummy_mips_map[1].virt = 
    (unsigned long)ioremap(dummy_mips_map[1].phys,
                           dummy_mips_map[1].size);
  simple_map_init(&dummy_mips_map[1]);
  mymtd[1] = do_map_probe("cfi_probe", &dummy_mips_map[1]);
  if(mymtd[1])
    mymtd[1]->owner = THIS_MODULE;
  if (!mymtd[0] || !mymtd[1])
    return -ENXIO;
 
 /* 
  * Now we will partition the boot flash. We are interested in the          
  * new mtd object for the second partition since we will be 
  * concatenating it with the other flash. 
  */
  boot_flash_partitions[1].mtdp = &concat_partitions[0];
  add_mtd_partitions(mymtd[0], boot_flash_partitions, 2);

  /* 
   * concat_partitions[1] should contain the mtd_info pointer for the 
   * 2nd partiton. Do the concatenation 
   */
  concat_partitions[1] = mymtd[1];
  concat_mtd = mtd_concat_create(concat_partitions, 2, 
                                 "JFFS2 flash concatenate");
  if(concat_mtd)
    add_mtd_device(concat_mtd);
  return 0;
}


static void __exit cleanup_dummy_mips_mtd_bsp(void)
{
  mtd_concat_destroy(concat_mtd);
  del_mtd_partitions(mymtd[0]);
  map_destroy(mymtd[0]);
  map_destroy(mymtd[1]);
}

module_init (init_dummy_mips_mtd_bsp);
module_exit (cleanup_dummy_mips_mtd_bsp);
MODULE_LICENSE ("GPL");
MODULE_AUTHOR ("Embedded Linux book");
MODULE_DESCRIPTION ("Sample Mapping driver");
